<?php

/**** FORM: select campaign functions ****/

function connect_node_functions_form() {
  drupal_add_css(drupal_get_path('module', 'connect') .'/connect.css');
  $child  = NULL;
  $p_nid   = arg(1);
  //$p_nid  = connect_nid();
  $parent = node_load($p_nid);
  $form   = array();

  $form['parent_id'] = array(
    '#type' => 'value',
    '#value' =>   $p_nid,
  );

  /*
   *choose functions
   */
  $form['connect_actions'] = array(
    '#type' => 'fieldset',
    '#title' => 'Choose functions to apply to this campaign',
  );
  $actions     = connect_get_actions($p_nid);
  $action_list = connect_call_hooks('describe', $parent, $child, 'parent');

  unset($action_list['connect_action_basic']);

  $options = array();
  foreach ($action_list as $action) {
    $options[$action['name']] = $action['title'] . '<p class="connect-comment">' . $action['desc'] . '</p>';
  }
  $form['connect_actions']['actions'] = array(
    '#type'    => 'checkboxes',
    '#title'   => '',
    '#options' => $options,
    '#default_value' => $actions,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
    );

  return $form;
}

function connect_node_functions_form_submit($form_id, $form_values) {
  // save action options
  $options = array();
  foreach( $form_values['actions'] as $key=>$value ) {
    if ("$key" == "$value") {
      $options[] = $value;
    }
  }

  // restore mandatory action
  $options[] = 'connect_action_basic';

  connect_node_options( $form_values['parent_id'], 'connect_actions', $options );

  drupal_set_message('Your selected functions have been updated.');
  drupal_set_message('If you have added new functions, please configure them from the "Settings" tab.');
}


/**** FORM: function settings ****/

function connect_node_settings_form() {
  drupal_add_css(drupal_get_path('module', 'connect') .'/connect.css');
  $child   = NULL;
  $p_nid   = arg(1);
  $parent  = node_load($p_nid);
  $form    = array();

  $form['parent_id'] = array(
    '#type' => 'value',
    '#value' =>   $p_nid,
  );

  /*
   * map variables
   */
  $map      = connect_get_map($p_nid);
  $required = connect_get_required_vars($parent);

  // variables
  if ( !empty($required['variables']) ) {
    $form['campaign_variables'] = array('#tree' => TRUE);
    foreach ( $required['variables'] as $action=>$vars ) {
      $description = $action('describe');
      $form['campaign_variables']["variables_$action"] = array(
        '#type'    => 'fieldset',
        '#title'   => $description['title'],
      );
      foreach( $vars as $key=>$formitem ) {
        $form['campaign_variables']["variables_$action"][$key] = $formitem;
      }
      if ($action == 'connect_action_basic') {
        $form['campaign_variables']["variables_$action"]['#weight'] = -1;
      }
    }
  }

  // parent node fields
  if ( !empty($required['parent']) ) {
    $options = connect_get_node_fields($parent->type);
    $form['variables_parent'] = array(
      '#type'    => 'fieldset',
      '#title'   => 'Required fields in parent/campaign node',
      '#tree'    => TRUE,
    );
    $form['variables_parent']['message'] = array(
      '#value'   => '<em>'. t('The selected functions store or use information from the campaign/parent node. Please identify which fields in the parent node correspond to the function settings below.') .'</em>',
    );
    foreach ( $required['parent'] as $key=>$desc ) {
      $form['variables_parent'][$key] = array(
        '#type'    => 'select',
        '#title'   => $desc,
        '#options' => $options,
        '#default_value' => isset($map[$key]) ? $map[$key]: '',
        '#required' => TRUE,
      );
    }
  }

  // child node fields
  $form['variables_child'] = array(
    '#type'    => 'fieldset',
    '#title'   => 'Required fields in child/participant node',
    '#tree'    => TRUE,
  );
  if ( connect_node_options( $parent->nid, 'participant_type' ) ) {
    if ( !empty($required['child']) ) {
      $form['variables_child']['message'] = array(
        '#value'   => '<em>'. t('The selected functions store or use information from the participant/child node. Please identify which fields in the child node correspond to the function settings below.') .'</em>',
      );
      $child_type = connect_node_options( $parent->nid, 'participant_type' );
      $options    = connect_get_node_fields($child_type);
      foreach ( $required['child'] as $key=>$desc ) {
        $form['variables_child'][$key] = array(
          '#type'    => 'select',
          '#title'   => $desc,
          '#options' => $options,
          '#default_value' => isset($map[$key]) ? $map[$key]: '',
          '#required' => TRUE,
        );
      }
    }
  }
  else {
    $form['variables_child']['message'] = array(
      '#value'   => t('Please select a participant type and return here to set up your variables.'),
    );
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
    );

  return $form;
}

// TODO: add settings validation hook

function connect_node_settings_form_submit($form_id, $form_values) {
  // save overall settings
  foreach ( $form_values['campaign_variables'] as $function=>$vars) {
    foreach ($vars as $key=>$value) {
      connect_node_options( $form_values['parent_id'], $key, $value );
    }
  }

  // save variable->field mapping
  $options = array();
  foreach ( array('variables_parent','variables_child') as $formitem) {
    if (isset($form_values[$formitem])) {
      foreach ($form_values[$formitem] as $key=>$value) {
        if ( !empty($value) ) {
          $options[$key] = $value;
        }
      }
    }
  }
  connect_node_options( $form_values['parent_id'], 'connect_map', $options );

  drupal_set_message('The campaign configuration has been updated.');
}

/**** FORM: connect module administration ****/

function connect_admin_form() {
  // grab all node types, format for use in form
  $sql = "SELECT type FROM {node_type};";
  $result = db_query($sql);
  while ($row = db_fetch_object($result)) {
    $type_options[$row->type] = $row->type;
  }

  // remove some standard node types
  $remove = array('blog', 'forum', 'page', 'story');
  $type_options = array_diff($type_options, $remove);

  $form = array();
   // participant node types
   $form['connect_participant_nodes'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Which node types hold participant information?'),
    '#options' => $type_options,
    '#required' => TRUE,
    '#default_value' => variable_get('connect_participant_nodes', array()),
    );

   // parent node types
   $form['connect_parent_nodes'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Which node types can be used as parent nodes?'),
    '#options' => $type_options,
    '#required' => TRUE,
    '#default_value' => variable_get('connect_parent_nodes', array()),
    );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
    );
  return $form;
}

function connect_admin_form_validate($form_id, $form_values) {
  $set_error = FALSE;
  $test = array_intersect($form_values['connect_participant_nodes'], $form_values['connect_parent_nodes']);
  foreach ($test as $type=>$value) {
    if ("$type" == "$value") {
      $set_error = TRUE;
    }
  }
  
//var_dump($test);

  if ($set_error) {
    form_set_error('', 'A node type cannot be both a parent and a child node.');
  }
}

function connect_admin_form_submit($form_id, $form_values) {
  // save participant node info
  $data = array();
  foreach ($form_values['connect_participant_nodes'] as $key=>$value) {
    if ($key === $value) {
      $data[] = $value;
    }
  }
  if (! empty($data)) {
    variable_set('connect_participant_nodes', $data);
    drupal_set_message(t('The participant node types have been updated.'));
  }

  // save parent node info
  $data = array();
  foreach ($form_values['connect_parent_nodes'] as $key=>$value) {
    if ($key === $value) {
      $data[] = $value;
    }
  }
  if (! empty($data)) {
    variable_set('connect_parent_nodes', $data);
    drupal_set_message(t('The parent node types have been updated.'));
  }
}
