<?php

/**
 * interface function for returning lookup description or data
 */
function connect_target_lookup( $op = 'lookup', $parent=NULL, $child=NULL, $type='' ) {
  $return = false;
  if ( $parent && $type ) {
    $lookup = connect_node_options( $parent->nid, $type.'_lookup_type' );
    if ($lookup) {
      eval('$return = '.$lookup.'($op, $parent, $child);');
    }
  }
  return $return;
}


/*
 * connect_lookup_NAME( $op = 'lookup', $child )
 *
 * @ param $op
 * 'lookup' -> perform the lookup
 * 'requires' -> get info to pass through via connect_action_hook function
 * 'describe' -> returns a title to display on the node settings form
 *
 * @ return
 * if $op=='lookup': object( object->email object->name object->fax, etc )
 * if $op=='requires' : array as per connect_action_* 'requires' op
 * if $op=='describe' : string describing the function
 */


/*
 *  function to look up contact info for Canadian MP -- basic lookup from Postal Code
 *
 *  uses the OpenConcept Consulting web service at http://makethechange.ca/
 *  which requires a client key
 *
 */
function connect_lookup_mp($op='lookup', &$parent=NULL, &$child = NULL) {
  switch ($op) {
    case 'describe' :
      return 'Canadian MP (postal code)';
      break;

    case 'requires':
      $return['variables']  = array(
        'makethechange_key' => array(
          '#type'  => 'textfield',
          '#title' => 'Makethechange web service key',
          '#default_value' => connect_node_options( $parent->nid, 'makethechange_key' ),
          '#required' => TRUE,
        ),
      );
      $return['child']  = array(
        'postal_code' => 'Postal code',
      );
      return $return;

    case 'validate' :
      $code = connect_value( 'postal_code', $parent, $child, 'child' );
      if ( !connect_is_postalcode($code) ) {
        form_set_error('', 'Please enter a valid postal code.');
      }
      break;

    case 'lookup':
      $mtc_key = connect_node_options( $parent->nid, 'makethechange_key' );
      $code = connect_value( 'postal_code', $parent, $child, 'child' );
      $edid = _connect_get_riding($mtc_key, $code, 'federal');
      if ($edid) {
        // *** Danger! in rare cases there may be > 1 matches: this
        // just grabs the first one; if this is a problem,
        // use the CCK MPautocomplete version to allow users to select a riding
        // or write a two-stage version of the participation form
        $mp = _connect_get_MP_by_EDID($code, $edid[0]['id']);
        if ($mp){
          $return['name']  = $mp->mp_name;
          $return['email'] = $mp->email;
          $return['fax']   = $mp->fax;
          return $return;
        }
      }
      break;
  }
}

/*
 *  function to look up contact info for Canadian MP -- Autocomplete version
 * requires MP autocomplete and riding selection fields
 *
 *  uses the OpenConcept Consulting web service at http://makethechange.ca/
 *  which requires a client key
 *
 */
function connect_lookup_mp_autocomplete($op='lookup', &$parent=NULL, &$child = NULL) {
  switch ($op) {
    case 'describe' :
      return 'Canadian MP (autocomplete + riding list)';
      break;

    case 'requires':
      $return['variables']  = array(
        'makethechange_key' => array(
          '#type'  => 'textfield',
          '#title' => 'Makethechange web service key',
          '#default_value' => connect_node_options( $parent->nid, 'makethechange_key' ),
          '#required' => TRUE,
        ),
      );
      $return['child']  = array(
        'riding_auto' => 'MP autocomplete field',
        'riding_list' => 'Riding list (to be selected if autocomplete fails)',
      );
      return $return;

    case 'validate' :
      $map  = connect_get_map($parent->nid);
      $riding_auto = $child[$map['riding_auto']][0]['value'];
      $riding_list = $child[$map['riding_list']]['key'];
      if ( empty($riding_auto) && empty($riding_list) ) {
        form_set_error('', 'Please enter your postal code or select your riding from the list.');
      }
      break;

    case 'lookup':
      $edid = 0;
      $riding_auto = connect_value( 'riding_auto', $parent, $child, 'child' );
      $riding_list = connect_value( 'riding_list', $parent, $child, 'child' );
      if ( !empty($riding_list) ) {
        $edid = (int) $riding_list;
      }
      if ( $edid == 0 && !empty($riding_auto) ) {
        list($id,$discard) = sscanf($riding_auto, "%d | %s");
        $edid = (int) $id;
      }

      // do the lookup
      if ( is_numeric($edid) ) {
        $mtc_key = connect_node_options( $parent->nid, 'makethechange_key' );
        $mp = _connect_get_MP_by_EDID( $mtc_key, $edid);
        if ($mp ){
          $return['name']  = $mp->mp_name;
          $return['email'] = $mp->email;
          $return['fax']   = $mp->fax;
          return $return;
        }
      } else {
        die('problem!');
      }
      return null;
  }
}

/*
 *  function to look up contact info for Ontario MPP
 *
 *  uses the OpenConcept Consulting web service at http://makethechange.ca/
 *  which requires a client key
 *
 */
/*
function connect_lookup_on_mpp(&$participant) {
  // validate postal code
  $code = $participant->field_postal_code[0][value];
  if (! connect_is_postalcode($code)) {
    form_set_error('', t('There was a problem with the form data.'));
    return false;
  }

  // lookup MPP info
  $service_key = connect_get_service_key();

  // cacheable
  $cached_address = cache_get('on_mpp_' . $code, 'cache_connect');
  if ($cached_address) {
    return $cached_address;
  }

  $get_url     = "http://makethechange.ca/provincial/on_riding.php?key=".urlencode($service_key)."&pc=".urlencode($code) . "&type=csv";

  $fh = fopen($get_url, "r") or watchdog('debug','fopen failed: ' . $get_url );
  $mpp_address = fread($fh, 200) or watchdog('debug','fread failed: ' . $get_url );
  fclose($fh);

  if (strpos($mpp_address, 'Error:')) {
    $mpp_address = false;
  }
  cache_set('on_mpp_' . $code, 'cache_connect',  $mpp_address, CACHE_PERMANENT);
  return $mpp_address;
}
*/


/*****  Makethechange functions *****/

// MP lookup using EDID
// TODO -- add caching
function _connect_get_MP_by_EDID($service_key = null, $edid = null) {
  if ( $service_key && is_numeric($edid) ) {
    // cached?
    //$cached_address = cache_get('mp_' . $edid, 'cache_connect');
    //if ($cached_address) {
    //  return $cached_address;
    //}

    // do lookup
    $mp_url  = 'http://makethechange.ca/federal/riding.php?type=json&key='.urlencode($service_key).'&edid='.$edid;
    $mp_info = trim( file_get_contents($mp_url , FALSE, NULL, 0, 8192) );
    // only cache + return if no error
    if ( !strpos($mp_info, 'Error:') ) {
      //cache_set('mp_' . $edid, 'cache_connect',  $mp_address, CACHE_PERMANENT);
      return json_decode( $mp_info );
    }
  }
  // parameter and lookup errors fall through here
  return null;
}


/*
 *  generic riding lookup
 *
 *  @parameters
 *    postalcode (string)
 *    scope (string)  : 'federal'|'provincial'
 *    detail (string) : 'ontario', 'ottawa', etc.
 *
 *  @return
 *    (array) of riding info = array( 'id'=>'', 'en'=>'', 'fr'=>'' )
 *
 *   TODO - fix the web service API.
 *
 */
function _connect_get_riding( $service_key, $postalcode = null, $scope='federal', $detail = 'federal') {
  // determine lookup URI based on type of lookup
  $scope_function = array (
    'federal'    => array ( 'federal' => 'pc2csv/index' ),
    'provincial' => array ( 'ontario' => 'provincial/on_riding' ),
    'municipal'  => array (),
  );

  if ( $service_key && connect_is_postalcode($postalcode) && isset($scope_function[$scope][$detail]) ) {
    $riding_url  = 'http://makethechange.ca/' .$scope_function[$scope][$detail]. '.php?key='.urlencode($service_key).'&pc='.urlencode($postalcode).'&type=riding';
    $fh = fopen($riding_url, "r") or watchdog('debug','fopen failed: ' . $riding_url );
    $riding_data = fread($fh, 200) or watchdog('debug','fread failed: ' . $riding_url );
    fclose($fh);

    // if there are results, walk through them and create an array of arrays
    if ( !empty($riding_data) ) {
      $row_token = strtok($riding_data, "\n");
      while ($row_token != false) {
        $return[] = connect_riding_data($row_token);
        $row_token = strtok("\n");
      }
      return $return;
    }
  }
  // parameter and lookup errors fall through here
  return null;
}


// turns riding data returned from web service into an indexed array
function connect_riding_data($data = NULL) {
  $return     = null;
  $array_keys = array('id','en','fr');
  $array_temp = explode(',',$data);
  for ($i = 0; $i < count($array_temp); $i++) {
    $return[$array_keys[$i]] = trim( $array_temp[$i] );
  }
  return $return;
}



/**** Utility functions ****/


/*
 *  Determine available target lookup types, return in select list format
 */
function connect_get_lookup_types( $select = FALSE) {
  static $return = array();
  if ( empty($return) ) {
    $functions = get_defined_functions();
    $array = $functions['user'];
    foreach ($array as $key=>$name ) {
      if ( strpos($name, 'connect_lookup_') !== FALSE ) {
        eval( '$label = '.$name.'(\'describe\');' );
        $return[$name] = $label;
      }
    }
    $return[0] = '';
    asort($return);
  }
  return $return;
}